/**
  * @file BeerLambert.cpp
  * @author E.Casellas-(The RECORD team -INRA )
  *
  * Copyright (C) 2009 INRA
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  */

#include <vle/extension/DifferenceEquation.hpp>
#include <vle/devs/DynamicsDbg.hpp>

namespace vd = vle::devs;
namespace ve = vle::extension;
namespace vv = vle::value;

namespace gluePhysic {
	class BeerLambert : public ve::DifferenceEquation::Multiple
	{
	public:
		BeerLambert(
		   const vd::DynamicsInit& atom,
		   const vd::InitEventList& evts)
		    : ve::DifferenceEquation::Multiple(atom, evts)
		{
			maxEff = (evts.exist("maxEff")) ? vv::toDouble(evts.get("maxEff")) : 1.0;  
			k = (evts.exist("k")) ? vv::toDouble(evts.get("k")) : 0.7;  
			
		    FractionIntercepted_Plant = createVar("FractionIntercepted_Plant");
		    FractionIntercepted_Soil = createVar("FractionIntercepted_Soil");
		    MaxEff = createVar("MaxEff");
		    K = createVar("K");
		    
		    LAI = createSync("LAI");
		}

		virtual ~BeerLambert() {};

		virtual void compute(const vd::Time& /*time*/)
		{
			MaxEff = MaxEff(-1);
			K = K(-1);
			
			FractionIntercepted_Plant = MaxEff() * (1 - exp(-K() * LAI()));
			FractionIntercepted_Soil = 1 - FractionIntercepted_Plant();
		}


		virtual void initValue(const vd::Time& /*time*/)
		{
			MaxEff = maxEff;
			K = k;
			
			FractionIntercepted_Plant = 0.0;
			FractionIntercepted_Soil = 1.0;
		}

	private:
		double maxEff;
		double k;
		
		Var FractionIntercepted_Plant;
		Var FractionIntercepted_Soil;
		Var MaxEff;
		Var K;
		
		Sync LAI;
	};
} // namespace gluePhysic
DECLARE_DYNAMICS_DBG(gluePhysic::BeerLambert)
