/**
  * @file DecFSAbl.cpp
  * @author E.Casellas-(The RECORD team -INRA )
  *
  * Copyright (C) 2009 INRA
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
  
#include <vle/extension/fsa/Statechart.hpp>
#include <vle/extension/DifferenceEquation.hpp>
#include <vle/utils/Trace.hpp>
#include <vle/utils/i18n.hpp>
#include <vle/devs/DynamicsDbg.hpp>

namespace ve = vle::extension;
namespace vd = vle::devs;

using namespace std;
using namespace vle;

namespace gluePhysic {
	enum DecState {DEC_STATE1, DEC_STATE2, DEC_STATE3};
	class DecFSAbl: public vle::extension::fsa::Statechart
	{
	public:
		DecFSAbl(const devs::DynamicsInit& atom,
				const devs::InitEventList& events) :
				Statechart(atom, events), lai(0)
		{
		    states(this) << DEC_STATE1 << DEC_STATE2 << DEC_STATE3;

			transition(this, DEC_STATE1, DEC_STATE2)
				<< guard(&DecFSAbl::cond1)
				<< send(&DecFSAbl::out_1);
		    transition(this, DEC_STATE2, DEC_STATE3)
		        << guard(&DecFSAbl::cond2)
		        << send(&DecFSAbl::out_2);

		    initialState(DEC_STATE1);        
		    
		   	 eventInState(this, "LAI", &DecFSAbl::inLai)
			>> DEC_STATE1 >> DEC_STATE2 >> DEC_STATE3;
		}
		virtual ~DecFSAbl() {};
		
		virtual vle::value::Value* observation(const vd::ObservationEvent& /*event*/) const
		{ return vle::value::Integer::create(currentState()); }
		
	private :
		//First transition 
		bool cond1(const vd::Time& julianDay)
		{ return (julianDay == 30); }
		
		void out_1(const vd::Time& /*julianDay*/, vd::ExternalEventList& output) const
		{
		    output << (ve::DifferenceEquation::Var("MaxEff") = 0.7);
		    output << (ve::DifferenceEquation::Var("K") = 0.9);
		}
		
		//Second transition
		bool cond2(const vd::Time& julianDay)
		{ return (julianDay == 300); }
		
		void out_2(const vd::Time& /*julianDay*/, vd::ExternalEventList& output) const
		{
		    output << (ve::DifferenceEquation::Var("MaxEff") = 0.5);
		    output << (ve::DifferenceEquation::Var("K") = 0.5);
		}

	   void inLai(const vd::Time& /* time */, const vd::ExternalEvent* event)
	   { lai << ve::DifferenceEquation::Var("LAI", event); }

		//consition variables
		double lai;
	};
}
DECLARE_DYNAMICS_DBG(gluePhysic::DecFSAbl);
