/**
  * @file LayerHierarchy.cpp
  * @author E.Casellas-(The RECORD team -INRA )
  *
  * Copyright (C) 2009 INRA
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation, either version 3 of the License, or
  * (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  *
  */

#include <vle/extension/DifferenceEquation.hpp>
#include <vle/devs/DynamicsDbg.hpp>

namespace vd = vle::devs;
namespace ve = vle::extension;
namespace vv = vle::value;

namespace gluePhysic {
	class LayerHierarchy : public ve::DifferenceEquation::Multiple
	{
	public:
		LayerHierarchy(
		   const vd::DynamicsInit& atom,
		   const vd::InitEventList& evts)
		    : ve::DifferenceEquation::Multiple(atom, evts)
		{
		    RadiationIntercepted_Layer1 = createVar("RadiationIntercepted_Layer1");
		    RadiationIntercepted_Layer2 = createVar("RadiationIntercepted_Layer2");
		    RadiationIntercepted_Layer3 = createVar("RadiationIntercepted_Layer3");
		    RadiationIntercepted_Layer4 = createVar("RadiationIntercepted_Layer4");
		    RadiationIntercepted_Soil = createVar("RadiationIntercepted_Soil");
		    
		    RG = createSync("RG");
		    FractionIntercepted_Layer1 = createSync("FractionIntercepted_Layer1");
		    FractionIntercepted_Layer2 = createSync("FractionIntercepted_Layer2");
		    FractionIntercepted_Layer3 = createSync("FractionIntercepted_Layer3");
		    FractionIntercepted_Layer4 = createSync("FractionIntercepted_Layer4");
		}

		virtual ~LayerHierarchy() {};

		virtual void compute(const vd::Time& /*time*/)
		{
			double residualRG;
			RadiationIntercepted_Layer1 = RG() * FractionIntercepted_Layer1();
			residualRG = RG() * ( 1 - FractionIntercepted_Layer1() );
		
			RadiationIntercepted_Layer2 = residualRG * FractionIntercepted_Layer2();
			residualRG = residualRG * ( 1 - FractionIntercepted_Layer2() );
		
			RadiationIntercepted_Layer3 = residualRG * FractionIntercepted_Layer3();
			residualRG = residualRG * ( 1 - FractionIntercepted_Layer3() );
		
			RadiationIntercepted_Layer4 = residualRG * FractionIntercepted_Layer4();
			residualRG = residualRG * ( 1 - FractionIntercepted_Layer4() );
		
			RadiationIntercepted_Soil = residualRG;
		}


		virtual void initValue(const vd::Time& /*time*/)
		{
			RadiationIntercepted_Layer1 = 0.0;	
			RadiationIntercepted_Layer2 = 0.0;	
			RadiationIntercepted_Layer3 = 0.0;	
			RadiationIntercepted_Layer4 = 0.0;	
			RadiationIntercepted_Soil = RG();	
		}


	private:
		Var RadiationIntercepted_Layer1;
		Var RadiationIntercepted_Layer2;
		Var RadiationIntercepted_Layer3;
		Var RadiationIntercepted_Layer4;
		Var RadiationIntercepted_Soil;
		
		Sync RG;
		Sync FractionIntercepted_Layer1;
		Sync FractionIntercepted_Layer2;
		Sync FractionIntercepted_Layer3;
		Sync FractionIntercepted_Layer4;
	};
} // namespace gluePhysic
DECLARE_DYNAMICS_DBG(gluePhysic::LayerHierarchy)
